@@ -166,6 +166,9 @@ Each entry is (path . timestamp).")
 (defvar vulpea-db-sync--poll-timer nil
   "Timer for polling-based external monitoring.")
 
+(defvar vulpea-db-sync--poll-scan-in-progress nil
+  "Non-nil when an async poll scan subprocess is running.")
+
 (defvar vulpea-db-sync--file-attributes (make-hash-table :test 'equal)
   "Cache of file attributes for external change detection.")
 
@@ -184,6 +187,9 @@ Reset when async processing completes.")
 (defvar vulpea-db-sync--sync-start-time nil
   "Start time of current sync phase.")
 
+(defvar vulpea-db-sync-debug nil
+  "When non-nil, log timing information for sync operations.")
+
 ;;; Mode
 
 ;;;###autoload
@@ -222,37 +228,114 @@ These are escaped by wrapping in brackets: * -> [*], ? -> [?], [ -> [[]]."
 (defun vulpea-db-sync--start ()
   "Start file watching and async update.
 
-Optionally performs initial scan based on `vulpea-db-sync-scan-on-enable'."
-  ;; Clean up deleted files first
-  (vulpea-db-sync--cleanup-deleted-files)
+Optionally performs initial scan based on `vulpea-db-sync-scan-on-enable'.
 
-  ;; Initial scan based on configuration
-  (when (and vulpea-db-sync-scan-on-enable vulpea-db-sync-directories)
-    (pcase vulpea-db-sync-scan-on-enable
-      ('async
-       ;; Queue all files for async processing
-       (message "Vulpea: Queueing files for async scan...")
-       (dolist (dir vulpea-db-sync-directories)
-         (dolist (file (vulpea-db-sync--list-org-files dir))
-           (vulpea-db-sync--enqueue file))))
-      ('blocking
-       ;; Scan synchronously (blocks Emacs)
-       (dolist (dir vulpea-db-sync-directories)
-         (vulpea-db-sync-update-directory dir)))))
+When `vulpea-db-sync-scan-on-enable' is `async', this function
+returns immediately without blocking.  File listing, cleanup of
+deleted files, and enqueueing are all performed asynchronously via
+a subprocess.  The `blocking' mode still scans synchronously."
+  (let ((t-total (current-time))
+        t-phase)
 
-  ;; Start watching directories if configured
-  (when vulpea-db-sync-directories
-    (dolist (dir vulpea-db-sync-directories)
-      (vulpea-db-sync--watch-directory dir)))
+    ;; Kill stale scan subprocess from previous activation
+    (when-let* ((proc (get-process "vulpea-scan")))
+      (delete-process proc))
 
-  ;; Start external monitoring (fswatch or polling)
-  (vulpea-db-sync--setup-external-monitoring)
+    ;; Start idle timer immediately (so queue is ready to process)
+    (unless vulpea-db-sync--idle-timer
+      (setq vulpea-db-sync--idle-timer
+            (run-with-idle-timer vulpea-db-sync-idle-delay t
+                                 #'vulpea-db-sync--process-queue)))
 
-  ;; Start idle timer for processing
-  (unless vulpea-db-sync--idle-timer
-    (setq vulpea-db-sync--idle-timer
-          (run-with-idle-timer vulpea-db-sync-idle-delay t
-                               #'vulpea-db-sync--process-queue))))
+    ;; Start external monitoring (fswatch is async, no blocking)
+    (setq t-phase (current-time))
+    (vulpea-db-sync--setup-external-monitoring)
+    (when vulpea-db-sync-debug
+      (message "[vulpea-sync] setup-external-monitoring: %.0fms"
+               (* 1000 (float-time (time-subtract (current-time) t-phase)))))
+
+    ;; Start watching directories via filenotify unless fswatch is
+    ;; active.  When fswatch is running it already monitors the
+    ;; filesystem for all changes (including in-Emacs saves), making
+    ;; filenotify redundant.  Programmatic changes (vulpea-create,
+    ;; vulpea-utils-with-note-sync) call vulpea-db-update-file
+    ;; directly and never rely on filenotify.
+    (setq t-phase (current-time))
+    (let ((watcher-count 0))
+      (if vulpea-db-sync--fswatch-process
+          (when vulpea-db-sync-debug
+            (message "[vulpea-sync] watch-directory: skipped (fswatch active)"))
+        (when vulpea-db-sync-directories
+          (dolist (dir vulpea-db-sync-directories)
+            (vulpea-db-sync--watch-directory dir))
+          (setq watcher-count (length vulpea-db-sync--watchers)))
+        (when vulpea-db-sync-debug
+          (message "[vulpea-sync] watch-directory: %.0fms (%d watchers)"
+                   (* 1000 (float-time (time-subtract (current-time) t-phase)))
+                   watcher-count))))
+
+    ;; Initial scan and cleanup based on configuration
+    (if (and vulpea-db-sync-scan-on-enable vulpea-db-sync-directories)
+        (pcase vulpea-db-sync-scan-on-enable
+          ('async
+           ;; Use subprocess to list files, then cleanup + enqueue
+           (when vulpea-db-sync-debug
+             (message "[vulpea-sync] launching async scan subprocess..."))
+           (let ((scan-start (current-time)))
+             (vulpea-db-sync--scan-files-async
+              vulpea-db-sync-directories
+              (lambda (files)
+                ;; Guard: skip if autosync was disabled while
+                ;; subprocess was running
+                (when vulpea-db-autosync-mode
+                  (when vulpea-db-sync-debug
+                    (message "[vulpea-sync] async scan found %d files in %.0fms"
+                             (length files)
+                             (* 1000 (float-time (time-subtract (current-time) scan-start)))))
+                  ;; Cleanup: remove DB entries not in the file list
+                  (let ((cleanup-start (current-time)))
+                    (vulpea-db-sync--cleanup-deleted-files-using files)
+                    (when vulpea-db-sync-debug
+                      (message "[vulpea-sync] async cleanup: %.0fms"
+                               (* 1000 (float-time (time-subtract (current-time) cleanup-start))))))
+                  ;; Enqueue all found files for change detection
+                  (let ((enqueue-start (current-time)))
+                    (dolist (file files)
+                      (vulpea-db-sync--enqueue file))
+                    (when vulpea-db-sync-debug
+                      (message "[vulpea-sync] async enqueue: %.0fms (%d files)"
+                               (* 1000 (float-time (time-subtract (current-time) enqueue-start)))
+                               (length files)))))))))
+          ('blocking
+           ;; Scan synchronously (blocks Emacs)
+           (setq t-phase (current-time))
+           (vulpea-db-sync--cleanup-deleted-files)
+           (when vulpea-db-sync-debug
+             (message "[vulpea-sync] cleanup-deleted-files: %.0fms"
+                      (* 1000 (float-time (time-subtract (current-time) t-phase)))))
+           (setq t-phase (current-time))
+           (dolist (dir vulpea-db-sync-directories)
+             (vulpea-db-sync-update-directory dir))
+           (when vulpea-db-sync-debug
+             (message "[vulpea-sync] blocking-scan: %.0fms"
+                      (* 1000 (float-time (time-subtract (current-time) t-phase)))))))
+      ;; No scan requested, but still cleanup deleted files
+      (setq t-phase (current-time))
+      (vulpea-db-sync--cleanup-deleted-files)
+      (when vulpea-db-sync-debug
+        (message "[vulpea-sync] cleanup-deleted-files: %.0fms"
+                 (* 1000 (float-time (time-subtract (current-time) t-phase))))))
+
+    ;; If schema was rebuilt, trigger forced re-index
+    (when vulpea-db--schema-rebuilt
+      (setq vulpea-db--schema-rebuilt nil)
+      (message "Vulpea: Schema upgraded, re-indexing all files...")
+      (dolist (dir vulpea-db-sync-directories)
+        (vulpea-db-sync-update-directory dir 'force)))
+
+    (when vulpea-db-sync-debug
+      (message "[vulpea-sync] start complete: %.0fms total (sync portion)"
+               (* 1000 (float-time (time-subtract (current-time) t-total)))))))
 
 (defun vulpea-db-sync--stop ()
   "Stop file watching and clear queue."
@@ -261,6 +344,10 @@ Optionally performs initial scan based on `vulpea-db-sync-scan-on-enable'."
     (file-notify-rm-watch (cdr entry)))
   (setq vulpea-db-sync--watchers nil)
 
+  ;; Stop async scan subprocess if running
+  (when-let* ((proc (get-process "vulpea-scan")))
+    (delete-process proc))
+
   ;; Stop external monitoring
   (vulpea-db-sync--stop-external-monitoring)
 
@@ -315,19 +402,80 @@ Optionally performs initial scan based on `vulpea-db-sync-scan-on-enable'."
   "Return non-nil when PATH points to a tracked org file.
 
 Excludes:
-- Non-.org files
+- Files not matching tracked extensions
 - Files in hidden directories (paths containing /.)"
   (and path
-       (string-suffix-p ".org" path)
+       (seq-some (lambda (ext) (string-suffix-p ext path))
+                 (vulpea-db--all-extensions))
        (not (string-match-p "/\\." path))))
 
 (defun vulpea-db-sync--list-org-files (dir)
   "List all tracked org files in DIR recursively.
 
 Uses `vulpea-db-sync--org-file-p' to filter files, ensuring
-consistency with file watcher filtering."
-  (seq-filter #'vulpea-db-sync--org-file-p
-              (directory-files-recursively dir "\\.org\\'")))
+consistency with file watcher filtering.
+
+DIR is expanded via `expand-file-name' to ensure returned paths
+are absolute (e.g., ~/notes becomes /home/user/notes).  This
+keeps paths consistent with `vulpea-db-sync--scan-files-async'
+and prevents tilde-vs-absolute mismatches in database queries."
+  (let ((dir (expand-file-name dir))
+        (regex (mapconcat (lambda (ext)
+                            (concat (regexp-quote ext) "\\'"))
+                          (vulpea-db--all-extensions)
+                          "\\|")))
+    (seq-filter #'vulpea-db-sync--org-file-p
+                (directory-files-recursively dir regex))))
+
+(defun vulpea-db-sync--scan-files-async (dirs callback)
+  "List org files in DIRS asynchronously, call CALLBACK with file list.
+
+Uses fd (or find as fallback) subprocess to avoid blocking Emacs.
+CALLBACK receives a list of absolute file paths."
+  (let* ((buffer "")
+         (dir (car dirs))
+         (expanded-dir (expand-file-name dir))
+         (extensions (vulpea-db--all-extensions))
+         (cmd (if (executable-find "fd")
+                  (append (list "fd" "--type" "f")
+                          (mapcan (lambda (ext)
+                                    (list "--extension" (substring ext 1)))
+                                  extensions)
+                          (list "--hidden" "--no-ignore"
+                                "--exclude" ".*"
+                                "." expanded-dir))
+                (let* ((name-args
+                        (mapcar (lambda (ext)
+                                  (list "-name" (concat "*" ext)))
+                                extensions))
+                       (name-clause
+                        (cl-loop for args in name-args
+                                 for first = t then nil
+                                 append (if first args (cons "-o" args)))))
+                  (append (list "find" expanded-dir "-type" "f")
+                          (if (> (length name-args) 1)
+                              (append '("(") name-clause '(")"))
+                            name-clause)
+                          (list "-not" "-path" "*/.*"))))))
+    (make-process
+     :name "vulpea-scan"
+     :command cmd
+     :connection-type 'pipe
+     :noquery t
+     :filter (lambda (_proc output)
+               (setq buffer (concat buffer output)))
+     :sentinel (lambda (_proc event)
+                 (when (string-prefix-p "finished" event)
+                   (let ((files (seq-filter
+                                 #'vulpea-db-sync--org-file-p
+                                 (split-string buffer "\n" t))))
+                     (if (cdr dirs)
+                         ;; More directories to scan
+                         (vulpea-db-sync--scan-files-async
+                          (cdr dirs)
+                          (lambda (more-files)
+                            (funcall callback (append files more-files))))
+                       (funcall callback files))))))))
 
 (defun vulpea-db-sync--drop-from-queue (path)
   "Remove PATH from the pending queue."
@@ -423,7 +571,8 @@ all files under that directory are removed."
              (not vulpea-db-sync--processing))
     (setq vulpea-db-sync--processing t)
     (unwind-protect
-        (let* ((batch-size (min (length vulpea-db-sync--queue)
+        (let* ((vulpea-db-sync--batch-start-time (current-time))
+               (batch-size (min (length vulpea-db-sync--queue)
                                 vulpea-db-sync-batch-size))
                (batch (seq-take vulpea-db-sync--queue batch-size))
                (paths (mapcar #'car batch))
@@ -478,6 +627,11 @@ all files under that directory are removed."
           (setq vulpea-db-sync--processed-total (+ vulpea-db-sync--processed-total updated unchanged)
                 vulpea-db-sync--updated-total (+ vulpea-db-sync--updated-total updated))
 
+          (when vulpea-db-sync-debug
+            (message "[vulpea-sync] batch: %.0fms (%d files, %d updated, %d unchanged)"
+                     (* 1000 (float-time (time-subtract (current-time) vulpea-db-sync--batch-start-time)))
+                     batch-size updated unchanged))
+
           ;; Report progress
           (when (and vulpea-db-sync-progress-interval
                      (> vulpea-db-sync--queue-total vulpea-db-sync-progress-interval)
@@ -576,6 +730,32 @@ Returns count of removed files."
                deleted (if (= deleted 1) "" "s")))
     deleted))
 
+(defun vulpea-db-sync--cleanup-deleted-files-using (existing-files)
+  "Remove DB entries for files not in EXISTING-FILES list.
+
+EXISTING-FILES is a list of absolute paths known to exist on disk
+\(typically from an fd/find subprocess).  This avoids per-file
+`file-exists-p' calls by comparing against the known set.
+
+Returns count of removed files."
+  (let* ((db (vulpea-db))
+         (existing-set (make-hash-table :test 'equal :size (length existing-files)))
+         (all-paths (mapcar #'car (emacsql db [:select path :from files])))
+         (deleted 0))
+    ;; Build set of existing files for O(1) lookup
+    (dolist (f existing-files)
+      (puthash f t existing-set))
+    (emacsql-with-transaction db
+      (dolist (path all-paths)
+        (unless (gethash path existing-set)
+          (vulpea-db--delete-file-notes path)
+          (emacsql db [:delete :from files :where (= path $s1)] path)
+          (setq deleted (1+ deleted)))))
+    (when (> deleted 0)
+      (message "Vulpea: Removed %d deleted file%s from database"
+               deleted (if (= deleted 1) "" "s")))
+    deleted))
+
 (defun vulpea-db-sync--cleanup-untracked-files ()
   "Remove database entries for files outside tracked directories.
 
@@ -825,7 +1005,7 @@ is enabled."
   (and path
        (file-name-absolute-p path)
        (seq-some (lambda (dir)
-                   (string-prefix-p (file-name-as-directory dir) path))
+                   (string-prefix-p (file-name-as-directory (expand-file-name dir)) path))
                  vulpea-db-sync-directories)))
 
 (defun vulpea-db-sync--fswatch-filter (_proc output)
@@ -918,27 +1098,42 @@ Handles partial lines by buffering incomplete output."
 (defun vulpea-db-sync--check-external-changes ()
   "Check for externally modified files by comparing mtimes.
 
+Uses fd (or find) subprocess to list files asynchronously, then
+compares modification times in the callback.  Skips if a previous
+scan is still in progress.
+
 Detects three types of changes:
 - New files: files not previously in the cache
 - Modified files: files with changed modification time
 - Deleted files: files in cache but no longer on disk"
+  (unless vulpea-db-sync--poll-scan-in-progress
+    (setq vulpea-db-sync--poll-scan-in-progress t)
+    (vulpea-db-sync--scan-files-async
+     vulpea-db-sync-directories
+     (lambda (files)
+       (unwind-protect
+           (vulpea-db-sync--check-external-changes-with-files files)
+         (setq vulpea-db-sync--poll-scan-in-progress nil))))))
+
+(defun vulpea-db-sync--check-external-changes-with-files (files)
+  "Compare FILES against cached attributes and enqueue modified ones.
+
+FILES is a list of org file paths from async directory scan."
   (let ((seen (make-hash-table :test 'equal)))
-    (dolist (dir vulpea-db-sync-directories)
-      (when (file-directory-p dir)
-        (dolist (file (vulpea-db-sync--list-org-files dir))
-          (when (file-exists-p file)
-            (let ((curr-attr (file-attributes file))
-                  (cached-attr (gethash file vulpea-db-sync--file-attributes)))
-              (puthash file curr-attr vulpea-db-sync--file-attributes)
-              (puthash file t seen)
-              (cond
-               ;; New file - not in cache before
-               ((not cached-attr)
-                (vulpea-db-sync--enqueue file))
-               ;; Modified file - mtime changed
-               ((not (equal (file-attribute-modification-time curr-attr)
-                            (file-attribute-modification-time cached-attr)))
-                (vulpea-db-sync--enqueue file))))))))
+    (dolist (file files)
+      (when (file-exists-p file)
+        (let ((curr-attr (file-attributes file))
+              (cached-attr (gethash file vulpea-db-sync--file-attributes)))
+          (puthash file curr-attr vulpea-db-sync--file-attributes)
+          (puthash file t seen)
+          (cond
+           ;; New file - not in cache before
+           ((not cached-attr)
+            (vulpea-db-sync--enqueue file))
+           ;; Modified file - mtime changed
+           ((not (equal (file-attribute-modification-time curr-attr)
+                        (file-attribute-modification-time cached-attr)))
+            (vulpea-db-sync--enqueue file))))))
     ;; Detect deletions - files in cache but not seen on disk
     (let (removed)
       (maphash (lambda (path _)
